@@ -48,30 +48,53 @@ package RT::CustomField;
 use strict;
 no warnings qw(redefine);
 
-use vars qw(@TYPES %TYPES $RIGHTS %FRIENDLY_OBJECT_TYPES);
+use vars qw(%FieldTypes $RIGHTS %FRIENDLY_OBJECT_TYPES);
 
 use RT::CustomFieldValues;
 use RT::ObjectCustomFieldValues;
 
-# Enumerate all valid types for this custom field
-@TYPES = (
-    'Freeform',	# loc
-    'Select',	# loc
-    'Text',     # loc
-    'Image',    # loc
-    'Binary',   # loc
+
+%FieldTypes = (
+    Select => [
+        'Select multiple values',	# loc
+        'Select one value',		# loc
+        'Select up to [_1] values',	# loc
+    ],
+    Freeform => [
+        'Enter multiple values',	# loc
+        'Enter one value',		# loc
+        'Enter up to [_1] values',	# loc
+    ],
+    Text => [
+        'Fill in multiple text areas',	# loc
+        'Fill in one text area',	# loc
+        'Fill in up to [_1] text areas',# loc
+    ],
+    Wikitext => [
+        'Fill in multiple wikitext areas',	# loc
+        'Fill in one wikitext area',	# loc
+        'Fill in up to [_1] wikitext areas',# loc
+    ],
+    Image => [
+        'Upload multiple images',	# loc
+        'Upload one image',		# loc
+        'Upload up to [_1] images',	# loc
+    ],
+    Binary => [
+        'Upload multiple files',	# loc
+        'Upload one file',		# loc
+        'Upload up to [_1] files',	# loc
+    ],
 );
 
-# Populate a hash of types of easier validation
-for (@TYPES) { $TYPES{$_} = 1};
 
+%FRIENDLY_OBJECT_TYPES =  ();
 
-%FRIENDLY_OBJECT_TYPES =  (
-    'RT::Queue-RT::Ticket'                 => "Tickets",		# loc
-    'RT::Queue-RT::Ticket-RT::Transaction' => "Ticket Transactions",	# loc
-    'RT::User'                             => "Users",			# loc
-    'RT::Group'                            => "Groups",			# loc
-);
+RT::CustomField->_ForObjectType( 'RT::Queue-RT::Ticket' => "Tickets", );    #loc
+RT::CustomField->_ForObjectType(
+    'RT::Queue-RT::Ticket-RT::Transaction' => "Ticket Transactions", );    #loc
+RT::CustomField->_ForObjectType( 'RT::User'  => "Users", );                           #loc
+RT::CustomField->_ForObjectType( 'RT::Group' => "Groups", );                          #loc
 
 $RIGHTS = {
     SeeCustomField            => 'See custom fields',       # loc_pair
@@ -191,6 +214,24 @@ sub Create {
     return $rv;
 }
 
+=head2 Load ID/NAME
+
+Load a custom field.  If the value handed in is an integer, load by custom field ID. Otherwise, Load by name.
+
+=cut
+
+
+sub Load {
+    my $self = shift;
+    my $id = shift;
+
+    if ($id =~ /^\d+$/) {
+        return ($self->SUPER::Load($id));
+    } else {
+        return($self->LoadByName(Name => $id));
+    }
+}
+
 
 # {{{ sub LoadByName
 
@@ -268,7 +309,8 @@ ok ($cf->SingleValue);
 is($cf->Type, 'Select');
 is($cf->MaxValues, 1);
 
-ok($cf->SetType('SelectMultiple'));
+my ($val, $msg) = $cf->SetMaxValues('0');
+ok($val, $msg);
 is($cf->Type, 'Select');
 is($cf->MaxValues, 0);
 ok(!$cf->SingleValue );
@@ -506,7 +548,7 @@ Retuns an array of the types of CustomField that are supported
 =cut
 
 sub Types {
-	return (@TYPES);
+	return (keys %FieldTypes);
 }
 
 # }}}
@@ -518,42 +560,13 @@ Returns a localized human-readable version of the custom field type.
 If a custom field type is specified as the parameter, the friendly type for that type will be returned
 
 =cut
-
-my %FriendlyTypes = (
-    Select => [
-        'Select multiple values',	# loc
-        'Select one value',		# loc
-        'Select up to [_1] values',	# loc
-    ],
-    Freeform => [
-        'Enter multiple values',	# loc
-        'Enter one value',		# loc
-        'Enter up to [_1] values',	# loc
-    ],
-    Text => [
-        'Fill in multiple text areas',	# loc
-        'Fill in one text area',	# loc
-        'Fill in up to [_1] text areas',# loc
-    ],
-    Image => [
-        'Upload multiple images',	# loc
-        'Upload one image',		# loc
-        'Upload up to [_1] images',	# loc
-    ],
-    Binary => [
-        'Upload multiple files',	# loc
-        'Upload one file',		# loc
-        'Upload up to [_1] files',	# loc
-    ],
-);
-
 sub FriendlyType {
     my $self = shift;
 
     my $type = @_ ? shift : $self->Type;
     my $max  = @_ ? shift : $self->MaxValues;
 
-    if (my $friendly_type = $FriendlyTypes{$type}[$max>2 ? 2 : $max]) {
+    if (my $friendly_type = $FieldTypes{$type}[$max>2 ? 2 : $max]) {
 	return ( $self->loc( $friendly_type, $max ) );
     }
     else {
@@ -592,7 +605,7 @@ sub ValidateType {
 	$RT::Logger->warning( "Prefix 'Single' and 'Multiple' to Type deprecated, use MaxValues instead");
     }
 
-    if( $TYPES{$type}) {
+    if( $FieldTypes{$type}) {
         return(1);
     }
     else {
@@ -1048,6 +1061,30 @@ sub ValuesForObject {
 }
 
 
+=head2 _ForObjectType PATH FRIENDLYNAME
+
+Tell RT that a certain object accepts custom fields
+
+Examples:
+
+    'RT::Queue-RT::Ticket'                 => "Tickets",		# loc
+    'RT::Queue-RT::Ticket-RT::Transaction' => "Ticket Transactions",	# loc
+    'RT::User'                             => "Users",			# loc
+    'RT::Group'                            => "Groups",			# loc
+
+This is a class method. 
+
+=cut
+
+sub _ForObjectType {
+    my $self = shift;
+    my $path = shift;
+    my $friendly_name = shift;
+
+    $FRIENDLY_OBJECT_TYPES{$path} = $friendly_name;
+
+}
+
 # }}}
 
 1;
