@@ -329,7 +329,7 @@ sub CreateTicket {
                 $ARGS{$arg} = [split('\n', $ARGS{$arg})];
             }
 
-            if ( $cf->Type eq 'Text') {
+            if ( $cf->Type =~ /text/i) { # Catch both Text and Wikitext
                 $ARGS{$arg} =~ s/\r//g;
             }
 
@@ -459,6 +459,19 @@ sub ProcessUpdateMessage {
             Body                => $args{ARGSRef}->{'UpdateContent'},
         );
 
+        if ($args{ARGSRef}->{'QuoteTransaction'} ) {
+            my $old_txn = RT::Transaction->new($session{'CurrentUser'}); 
+            $old_txn->Load($args{ARGSRef}->{'QuoteTransaction'} );
+            if ($old_txn->Message->First) {
+                $Message->head->replace('In-Reply-To',$old_txn->Message->First->GetHeader('Message-Id')); 
+                $Message->head->replace('References',
+                            $old_txn->Message->First->GetHeader('References')
+                            . " ".
+                            $old_txn->Message->First->GetHeader('Message-Id')
+                        ); 
+            }
+
+        }
         if ($args{ARGSRef}->{'UpdateAttachments'}) {
             $Message->make_multipart;
             $Message->add_part($_) foreach values %{$args{ARGSRef}->{'UpdateAttachments'}};
@@ -1140,7 +1153,7 @@ sub ProcessObjectCustomFieldUpdates {
 		    my @values = ();
 		    if (ref( $ARGSRef->{$arg} ) eq 'ARRAY' ) {
 			@values = @{ $ARGSRef->{$arg} };
-		    } elsif ($CustomFieldObj->Type eq 'Text') {
+		    } elsif ($CustomFieldObj->Type =~ /text/i) { # Both Text and Wikitext
 			@values = ($ARGSRef->{$arg});
 		    } else {
 			@values = split /\n/, $ARGSRef->{$arg};
@@ -1148,7 +1161,7 @@ sub ProcessObjectCustomFieldUpdates {
 		    
 		    if ( ($CustomFieldObj->Type eq 'Freeform' 
 			  && ! $CustomFieldObj->SingleValue) ||
-			  $CustomFieldObj->Type eq 'Text') {
+			  $CustomFieldObj->Type =~ /text/i) {
 			foreach my $val (@values) {
 			    $val =~ s/\r//g;
 			}
