@@ -124,6 +124,31 @@ sub Delete {
     } 
 }
 
+=head2 ObjectTypeStr
+
+Returns a string which is this object's type.  The type is the class,
+without the "RT::" prefix.
+
+=begin testing
+
+my $ticket = RT::Ticket->new($RT::SystemUser);
+my $group = RT::Group->new($RT::SystemUser);
+is($ticket->ObjectTypeStr, 'Ticket', "Ticket returns correct typestring");
+is($group->ObjectTypeStr, 'Group', "Group returns correct typestring");
+
+=end testing
+
+=cut
+
+sub ObjectTypeStr {
+    my $self = shift;
+    if (ref($self) =~ /^.*::(\w+)$/) {
+	return $self->loc($1);
+    } else {
+	return $self->loc(ref($self));
+    }
+}
+
 =head2 Attributes
 
 Return this object's attributes as an RT::Attributes object
@@ -474,12 +499,33 @@ sub _Set {
         $args{'Value'} = 0;
     }
 
-    $self->_SetLastUpdated();
-    my ( $val, $msg ) = $self->SUPER::_Set(
+    my $old_val = $self->__Value($args{'Field'});
+     $self->_SetLastUpdated();
+    my $ret = $self->SUPER::_Set(
         Field => $args{'Field'},
         Value => $args{'Value'},
         IsSQL => $args{'IsSQL'}
     );
+        my ($status, $msg) =  $ret->as_array();
+
+        # @values has two values, a status code and a message.
+
+    # $ret is a Class::ReturnValue object. as such, in a boolean context, it's a bool
+    # we want to change the standard "success" message
+    if ($status) {
+        $msg =
+          $self->loc(
+            "[_1] changed from [_2] to [_3]",
+            $args{'Field'},
+            ( $old_val ? "'$old_val'" : $self->loc("(no value)") ),
+            '"' . $self->__Value( $args{'Field'}) . '"' 
+          );
+      } else {
+
+          $msg = $self->CurrentUser->loc_fuzzy($msg);
+    }
+    return wantarray ? ($status, $msg) : $ret;     
+
 }
 
 # }}}
@@ -871,12 +917,12 @@ sub Update {
         next if ( $value eq $self->$attribute() );
         my $method = "Set$attribute";
         my ( $code, $msg ) = $self->$method($value);
-
         my ($prefix) = ref($self) =~ /RT::(\w+)/;
-        push @results,
-          $self->loc( "$prefix [_1]", $self->id ) . ': '
-          . $self->loc($attribute) . ': '
-          . $self->CurrentUser->loc_fuzzy($msg);
+
+        # Default to $id, but use name if we can get it.
+        my $label = $self->id;
+        $label = $self->Name if (UNIVERSAL::can($self,'Name'));
+        push @results, $self->loc( "$prefix [_1]", $label ) . ': '. $msg;
 
 =for loc
                                    "[_1] could not be set to [_2].",       # loc
@@ -1014,10 +1060,12 @@ ok (!$t2->HasUnresolvedDependencies, "Ticket ".$t2->Id." has no unresolved deps"
 
 my ($rid, $rmsg)= $t1->Resolve();
 ok(!$rid, $rmsg);
-ok($t2->Resolve);
+my ($rid2, $rmsg2) = $t2->Resolve();
+ok ($rid2, $rmsg2);
 ($rid, $rmsg)= $t1->Resolve();
 ok(!$rid, $rmsg);
-ok($t3->Resolve);
+my ($rid3,$rmsg3) = $t3->Resolve;
+ok ($rid3,$rmsg3);
 ($rid, $rmsg)= $t1->Resolve();
 ok($rid, $rmsg);
 
@@ -1443,14 +1491,14 @@ sub CustomFields {
     $cfs->UnLimit;
 
     # XXX handle multiple types properly
-    foreach my $lookup ($self->_LookupTypes) {
-	$cfs->LimitToLookupType($lookup);
-	$cfs->LimitToGlobalOrObjectId($self->_LookupId($lookup));
-    }
+	$cfs->LimitToLookupType($self->CustomFieldLookupType);
+	$cfs->LimitToGlobalOrObjectId($self->_LookupId($self->CustomFieldLookupType));
 
     return $cfs;
 }
 
+# TODO: This _only_ works for RT::Class classes. it doesn't work, for example, for RT::FM classes. 
+
 sub _LookupId {
     my $self = shift;
     my $lookup = shift;
@@ -1464,7 +1512,27 @@ sub _LookupId {
     return $self->Id;
 }
 
-sub _LookupTypes { ref($_[0]) }
+
+=head2 CustomFieldLookupType 
+
+Returns the path RT uses to figure out which custom fields apply to this object.
+
+=cut
+
+sub CustomFieldLookupType {
+    my $self = shift;
+    return ref($self);
+}
+
+#TODO Deprecated API. Destroy in 3.6
+sub _LookupTypes { 
+    my  $self = shift;
+    $RT::Logger->warning("_LookupTypes call is deprecated. Replace with CustomFieldLookupType");
+    $RT::Logger->warning("Besides, it was a private API. Were you doing using it?");
+
+    return($self->CustomFieldLookupType);
+
+}
 
 # {{{ AddCustomFieldValue
 
@@ -1496,6 +1564,10 @@ sub _AddCustomFieldValue {
     );
 
    # {{{ Get a custom field object from the 'Field' parameter.
+   #
+   # XXX TODO: RT should be looking for a custom field for this object 
+   # with this name, if it's a name, rather than just "Load"
+   #
     my $cf = RT::CustomField->new( $self->CurrentUser );
     if ( UNIVERSAL::isa( $args{'Field'}, "RT::CustomField" ) ) {
         $cf->Load( $args{'Field'}->id );
@@ -1754,7 +1826,7 @@ sub CustomFieldValues {
         # Look up the field ID.
         my $cfs = RT::CustomFields->new( $self->CurrentUser );
         $cfs->LimitToGlobalOrObjectId( $self->Id() );
-        $cfs->LimitToLookupType($self->_LookupTypes);
+        $cfs->LimitToLookupType($self->CustomFieldLookupType);
         $cfs->Limit( FIELD => 'Name', OPERATOR => '=', VALUE => $field );
 
         if ( $cfs->First ) {
