@@ -73,6 +73,10 @@
 =begin testing
 
 ok (require RT::Tickets);
+ok( my $testtickets = RT::Tickets->new( $RT::SystemUser ) );
+ok( $testtickets->LimitStatus( VALUE => 'deleted' ) );
+# Should be zero until 'allow_deleted_search'
+ok( $testtickets->Count == 0 );
 
 =end testing
 
@@ -114,30 +118,27 @@ my %FIELDS = (
     DependentOn     => [ 'LINK' => From => 'DependsOn', ],
     DependedOnBy    => [ 'LINK' => From => 'DependsOn', ],
     ReferredToBy    => [ 'LINK' => From => 'RefersTo', ],
-
-    #   HasDepender	    => ['LINK',],
-    #   RelatedTo	    => ['LINK',],
-    Told             => [ 'DATE'            => 'Told', ],
-    Starts           => [ 'DATE'            => 'Starts', ],
-    Started          => [ 'DATE'            => 'Started', ],
-    Due              => [ 'DATE'            => 'Due', ],
-    Resolved         => [ 'DATE'            => 'Resolved', ],
-    LastUpdated      => [ 'DATE'            => 'LastUpdated', ],
-    Created          => [ 'DATE'            => 'Created', ],
-    Subject          => [ 'STRING', ],
-    Content          => [ 'TRANSFIELD', ],
-    ContentType      => [ 'TRANSFIELD', ],
-    Filename         => [ 'TRANSFIELD', ],
-    TransactionDate  => [ 'TRANSDATE', ],
+    Told	    => ['DATE' => 'Told',],
+    Starts	    => ['DATE' => 'Starts',],
+    Started	    => ['DATE' => 'Started',],
+    Due		    => ['DATE' => 'Due',],
+    Resolved	    => ['DATE' => 'Resolved',],
+    LastUpdated	    => ['DATE' => 'LastUpdated',],
+    Created	    => ['DATE' => 'Created',],
+    Subject	    => ['STRING',],
+    Content	    => ['TRANSFIELD',],
+    ContentType	    => ['TRANSFIELD',],
+    Filename        => ['TRANSFIELD',],
+    TransactionDate => ['TRANSDATE',],
+    Requestor       => ['WATCHERFIELD' => 'Requestor',],
+    Requestors       => ['WATCHERFIELD' => 'Requestor',],
+    Cc              => ['WATCHERFIELD' => 'Cc',],
+    AdminCc         => ['WATCHERFIELD' => 'AdminCc',],
+    Watcher	    => ['WATCHERFIELD'],
+    LinkedTo	    => ['LINKFIELD',],
+    CustomFieldValue =>['CUSTOMFIELD',],
+    CF              => ['CUSTOMFIELD',],
     Updated          => [ 'TRANSDATE', ],
-    Requestor        => [ 'WATCHERFIELD'    => 'Requestor', ],
-    Requestors       => [ 'WATCHERFIELD'    => 'Requestor', ],
-    Cc               => [ 'WATCHERFIELD'    => 'Cc', ],
-    AdminCc          => [ 'WATCHERFIELD'    => 'AdminCC', ],
-    Watcher          => ['WATCHERFIELD'],
-    LinkedTo         => [ 'LINKFIELD', ],
-    CustomFieldValue => [ 'CUSTOMFIELD', ],
-    CF               => [ 'CUSTOMFIELD', ],
     RequestorGroup   => [ 'MEMBERSHIPFIELD' => 'Requestor', ],
     CCGroup          => [ 'MEMBERSHIPFIELD' => 'Cc', ],
     AdminCCGroup     => [ 'MEMBERSHIPFIELD' => 'AdminCc', ],
@@ -451,8 +452,8 @@ sub _DateLimit {
     require Time::ParseDate;
     use POSIX 'strftime';
 
-    # FIXME: Replace me with RT::Date( Type => 'unknown' ...)
-    my $time = Time::ParseDate::parsedate(
+    # XXX TODO FIXME: Replace me with RT::Date( Type => 'unknown' ...)
+    my $time =  Time::ParseDate::parsedate(
         $value,
         UK            => $RT::DateDayBeforeMonth,
         PREFER_PAST   => $RT::AmbiguousDayInPast,
@@ -1080,11 +1081,13 @@ sub _CustomFieldLimit {
     $field = $1 if $field =~ /^{(.+)}$/;    # trim { }
 
 
-# If we're trying to find custom fields that don't match something, we want tickets
-# where the custom field has no value at all
+# If we're trying to find custom fields that don't match something, we
+# want tickets where the custom field has no value at all.  Note that
+# we explicitly don't include the "IS NULL" case, since we would
+# otherwise end up with a redundant clause.
 
     my $null_columns_ok;
-    if ( ( $op =~ /^IS$/i ) or ( $op =~ /^NOT LIKE$/i ) or ( $op eq '!=' ) ) {
+    if ( ( $op =~ /^NOT LIKE$/i ) or ( $op eq '!=' ) ) {
         $null_columns_ok = 1;
     }
 
@@ -1108,19 +1111,50 @@ sub _CustomFieldLimit {
     }
 
     my $TicketCFs;
+    my $cfkey = $cfid ? $cfid : "$queue.$field";
 
     # Perform one Join per CustomField
-    if ( $self->{_sql_object_cf_alias}{$cfid} ) {
-        $TicketCFs = $self->{_sql_object_cf_alias}{$cfid};
+    if ( $self->{_sql_object_cf_alias}{$cfkey} ) {
+        $TicketCFs = $self->{_sql_object_cf_alias}{$cfkey};
     }
     else {
-        $TicketCFs = $self->{_sql_object_cf_alias}{$cfid} = $self->Join(
-            TYPE   => 'left',
-            ALIAS1 => 'main',
-            FIELD1 => 'id',
-            TABLE2 => 'ObjectCustomFieldValues',
-            FIELD2 => 'ObjectId'
-        );
+        if ($cfid) {
+            $TicketCFs = $self->{_sql_object_cf_alias}{$cfkey} = $self->Join(
+                TYPE   => 'left',
+                ALIAS1 => 'main',
+                FIELD1 => 'id',
+                TABLE2 => 'ObjectCustomFieldValues',
+                FIELD2 => 'ObjectId',
+            );
+            $self->SUPER::Limit(
+                LEFTJOIN        => $TicketCFs,
+                FIELD           => 'CustomField',
+                VALUE           => $cfid,
+                ENTRYAGGREGATOR => 'AND'
+            );
+        } else {
+            my $cfalias = $self->Join(
+                TYPE   => 'left',
+                EXPRESSION =>   "'$field'",
+                TABLE2 => 'CustomFields',
+                FIELD2 => 'Name',
+            );
+
+            $TicketCFs = $self->{_sql_object_cf_alias}{$cfkey} = $self->Join(
+                TYPE   => 'left',
+                ALIAS1 => $cfalias,
+                FIELD1 => 'id',
+                TABLE2 => 'ObjectCustomFieldValues',
+                FIELD2 => 'CustomField',
+            );
+            $self->SUPER::Limit(
+                LEFTJOIN => $TicketCFs,
+                FIELD => 'ObjectId',
+                VALUE => 'main.id',
+                QUOTEVALUE => 0,
+                ENTRYAGGREGATOR => 'AND',
+            );
+        }
         $self->SUPER::Limit(
             LEFTJOIN => $TicketCFs,
             FIELD    => 'ObjectType',
@@ -1134,29 +1168,6 @@ sub _CustomFieldLimit {
             OPERATOR    => '=',
             VALUE => '0',
             ENTRYAGGREGATOR => 'AND');
-
-        if ($cfid) {
-            $self->SUPER::Limit(
-                LEFTJOIN        => $TicketCFs,
-                FIELD           => 'CustomField',
-                VALUE           => $cfid,
-                ENTRYAGGREGATOR => 'AND'
-            );
-        }
-        else {
-            my $cfalias = $self->Join(
-                ALIAS1 => $TicketCFs,
-                FIELD1 => 'CustomField',
-                TABLE2 => 'CustomFields',
-                FIELD2 => 'id'
-            );
-            $self->SUPER::Limit(
-                LEFTJOIN => $cfalias,
-                FIELD    => 'Name',
-                VALUE    => $field,
-            );
-
-        }
     }
 
     $self->_OpenParen if ($null_columns_ok);
@@ -1169,6 +1180,7 @@ sub _CustomFieldLimit {
         QUOTEVALUE => 1,
         @rest
     );
+
     if ($null_columns_ok) {
         $self->_SQLLimit(
             ALIAS           => $TicketCFs,
@@ -1344,6 +1356,11 @@ Takes a paramhash with the fields OPERATOR and VALUE.
 OPERATOR is one of = or !=.
 VALUE is a status.
 
+RT adds Status != 'deleted' until object has
+allow_deleted_search internal property set.
+$tickets->{'allow_deleted_search'} = 1;
+$tickets->LimitStatus( VALUE => 'deleted' );
+
 =cut
 
 sub LimitStatus {
@@ -1947,7 +1964,7 @@ sub LimitReferredToBy {
     my $ticket_id = shift;
     $self->LimitLinkedFrom(
         BASE => "$ticket_id",
-        TYPE => 'ReferredTo',
+        TYPE => 'ReferredToBy',
     );
 
 }
@@ -2259,9 +2276,18 @@ sub Next {
     my $Ticket = $self->SUPER::Next();
     if ( ( defined($Ticket) ) and ( ref($Ticket) ) ) {
 
-        #Make sure we _never_ show deleted tickets
-        #TODO we should be doing this in the where clause.
-        #but you can't do multiple clauses on the same field just yet :/
+	    if ( $Ticket->__Value('Status') eq 'deleted' &&
+			!$self->{'allow_deleted_search'} ) {
+		return($self->Next());
+	    }
+            # Since Ticket could be granted with more rights instead
+            # of being revoked, it's ok if queue rights allow
+            # ShowTicket.  It seems need another query, but we have
+            # rights cache in Principal::HasRight.
+  	    elsif ($Ticket->QueueObj->CurrentUserHasRight('ShowTicket') ||
+                   $Ticket->CurrentUserHasRight('ShowTicket')) {
+		return($Ticket);
+	    }
 
         if ( $Ticket->__Value('Status') eq 'deleted' ) {
             return ( $self->Next() );
@@ -2595,5 +2621,24 @@ sub PrepForSerialization {
     $self->RedoSearch();
 }
 
+
+=head1 FLAGS
+
+RT::Tickets supports several flags which alter search behavior:
+
+
+allow_deleted_search  (Otherwise never show deleted tickets in search results)
+looking_at_type (otherwise limit to type=ticket)
+
+These flags are set by calling 
+
+$tickets->{'flagname'} = 1;
+
+BUG: There should be an API for this
+
+=cut
+
 1;
 
+
+
